#!/bin/sh
# Begin libiptables2.sh

include common

depends iptables

[ "$AUTH_PREFIX" == "" ] && AUTH_PREFIX="AUTH-"
[ "$DROP_PREFIX" == "" ] && DROP_PREFIX="DROP-"
[ "$MARK_PREFIX" == "" ] && MARK_PREFIX="MARK-"
[ "$JUMP_PREFIX" == "" ] && JUMP_PREFIX="JUMP-"
[ "$DNAT_PREFIX" == "" ] && DNAT_PREFIX="JUMP-"
[ "$LOG_PREFIX" == ""  ] &&  LOG_PREFIX="LOG-"

ACCEPTED_CHAINS="INPUT FORWARD OUTPUT"

function exist_chain()
{
	usage_string="usage: $FUNCNAME {chain} [table]"

        [ "$#" != "1" ] && [ "$#" != "2" ] && print_exit $usage_string

	[ "$2" == "" ] && table="" || table="-t $2"

	iptables $table -L $1 >/dev/null 2>&1
	return $?
};

function exist_table()
{
	usage_string="usage: $FUNCNAME {table}"

        [ "$#" != "1" ] && print_exit $usage_string
	
	iptables -t $1 -L > /dev/null 2>&1
	return $?
};

function init_table()
{
	usage_string="usage: $FUNCNAME {table} [tables ...]"

        [ "$#" == "0" ] && print_exit $usage_string
	
	for i in $*
	do
	  iptables -t $i -F > /dev/null 2>&1 || print_error "'$i' is not a recognized table"
	done
};

function masq()
{
    usage_string="usage: $FUNCNAME [log] [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_prefix="$2"
	log_yesno="$1"
	shift
	shift
    fi

    [ "$log_yesno" == "-log" ] &&
    iptables -t nat -A POSTROUTING $@ -m limit --limit 3/s -j LOG --log-prefix "$MASQ_PREFIX$log_prefix: "

    iptables -t nat -A POSTROUTING $@ -j MASQUERADE ||
    print_error "iptable returned error."
}

function soft_drop()
{
    usage_string="usage: $FUNCNAME {tables} [log] [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    Current_chains=""
    IFS=":"
    for i in $1
    do
      echo $ACCEPTED_CHAINS | grep $i > /dev/null &&
        Current_chains="$i "$Current_chains ||
	print_error "invalid CHAIN : '$i' in $FUNCNAME"
    done

    shift


    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_prefix="$2"
	log_yesno="$1"
	shift
	shift
    fi

    IFS=" "
    for i in $Current_chains
    do
    [ "$log_yesno" == "-log" ] &&
      iptables -A $i $@ -m limit --limit 3/s -j LOG --log-prefix "$DROP_PREFIX$log_prefix: "
      iptables -A $i $@ -m limit --limit 3/s -j REJECT &&
      iptables -A $i $@ -j DROP ||
      print_error "iptable returned error."
   done
}

function accept()
{
    usage_string="usage: $FUNCNAME {tables} [log] [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    Current_chains=""
    IFS=":"
    for i in $1
    do
      echo $ACCEPTED_CHAINS | grep $i > /dev/null &&
        Current_chains="$i "$Current_chains ||
	print_error "invalid CHAIN : '$i' in $FUNCNAME"
    done

    shift


    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_prefix="$2"
	log_yesno="$1"
	shift
	shift
    fi

    IFS=" "
    for i in $Current_chains
    do
    [ "$log_yesno" == "-log" ] &&
      iptables -A $i $@ -m limit --limit 3/s -j LOG --log-prefix "$AUTH_PREFIX$log_prefix: "
      iptables -A $i $@ -j ACCEPT || print_error "iptable returned error."
   done

}

function dnat()
{
    usage_string="usage: $FUNCNAME [log] {dest} [conditions]"

    [ "$#" != 0 ] || print_exit $usage_string

    log_yesno=""
    if [ "$1" == "-log" ] || [ "$1" == "-nolog" ]
    then
	log_prefix="$2"
	log_yesno="$1"
	shift
	shift
    fi

    dest_ip="$1"
    shift


    [ "$log_yesno" == "-log" ] &&
    iptables -t nat -A PREROUTING $@ -m limit --limit 3/s -j LOG --log-prefix "$MARK_DNAT$log_prefix: "

    iptables -t nat -A PREROUTING $@ -j DNAT --to-destination $dest_ip ||
    print_error "iptable returned error."
}


# End libiptables2.sh

